import pickle
import sys

from orgasm.indexer import Index

from backtranslate.fasta import fasta
import samples


def getIndex(config):
    '''
    
    @param config: a configuration object
    @return: an Indexer instance
    '''
    
    return Index(config.indexname)

def getProbes(config):
    '''
    According to the configuration file and the --seeds option this function return the set
    of sequence probes to use
    
     
    @param config: a configuration object
    @return: a dictionary with gene name as key and the 
             nucleic or protein sequence as value
    @rtype: dict
    '''
    try:
        p = fasta(config.seedname)
        config.logger.info("Load probe sequences from file : %s" % config.seedname)
    except IOError:
        p = getattr(samples,config.seedname)
        config.logger.info("Load probe internal dataset : %s" % config.seedname)
                    
    return p

def getSeeds(index,protein,config):
    # looks if the internal blast was already run            
             
    
                
    try:
        # yes, load the previous results
        seeds = pickle.load(open(config.outputname+'.omx'))
        config.logger.info("Load protein match from previous run : %d matches" % sum(len(seeds[i]) for i in seeds))
    except: 
        # no, run it and save the results
        config.logger.info("Running protein matching against reads...")
        seeds = index.lookForSeeds(protein,mincov=config.seedmincov)
        config.logger.info("==> %d matches" % sum(len(seeds[i]) for i in seeds))

        pickle.dump(seeds,open(config.outputname+'.omx',"w"))

    return seeds

#def reloadAssembling