#!/usr/bin/env python
################################################
# setenv PYTHONPATH ./orgasm/build/lib.macosx-10.6-intel-2.7/
# echo $PYTHONPATH
# ./orgasm/build/lib.macosx-10.6-intel-2.7/
# python orgasmb.py ./samples/GWM-261 ./samples/COX1_Machaon.fasta machaon

from orgasm.tango import tango, \
                         matchtoseed, \
                         cutLowCoverage, \
                         scaffold, \
                         coverageEstimate, \
                         fillGaps, \
                         path2fasta, \
                         unfoldAssembling, \
                         cutSNPs, \
                         selectGoodComponent, \
                         estimateDeadBrancheLength, pathConstraints,\
                         estimateFragmentLength
from orgasm.indexer import Index
from orgasm.backtranslate.fasta import fasta
from orgasm.assembler import Assembler
import orgasm.samples
                           
import sys
import pickle    

def estimateMinRead(index,minoverlap,coverage):
    MINREAD=10
    MINREADCOR=5
    MINOVERLAP=50
    minread =  (index.getReadSize() - minoverlap) * coverage / index.getReadSize()  / MINREADCOR
    if minread < MINREAD:
        minoverlap = index.getReadSize() - (MINREAD * MINREADCOR * index.getReadSize() / coverage)
        minread = MINREAD
    if  minoverlap< MINOVERLAP:
        minread =  MINREAD
        minoverlap = MINOVERLAP
    return minread,minoverlap

if __name__ == '__main__':
    
    import argparse
    
    # Declare script options
    
    parser = argparse.ArgumentParser(description='Assembly program dedicated to organelle genome reconstruction.')
    parser.add_argument('indexFilename', metavar='index', help='index root filename (produced by orgasmi)')
    
    parser.add_argument('seedsFilenameAA', metavar='seeds', help='protein seeds; either a file containing seeds proteic sequences or internal set of seeds among %s' % str(filter(lambda s: s.startswith('prot'),dir(orgasm.samples))) )

    parser.add_argument('outputFilename', metavar='output', help='output prefix')
    
    parser.add_argument('--minread', dest='minread', type=int, action='store', default=None, help='the minimum count of read to consider [default: <estimated>]')
    parser.add_argument('--minratio', dest='minratio', type=float, action='store', default=0.1, help='minimum ratio between occurrences of an extension and the occurrences of the most frequent extension to keep it. [default: %(default)f]')
    parser.add_argument('--mincov', dest='mincov', type=int, action='store', default=1, help='minimum occurrences of an extension to keep it. [default: %(default)d]')
    parser.add_argument('--minoverlap', dest='minoverlap', type=int, action='store', default=40, help='minimum length of the overlap between the sequence and reads to participate in the extension. [default: %(default)d (except if minread is estimated)]')
    parser.add_argument('--smallbranches', dest='smallbranches', type=int, action='store', default=None, help='maximum length of the branches to cut during the cleaning process [default: <estimated>]')
    parser.add_argument('--lowcomplexity', dest='lowcomplexity', action='store_true', default=False, help='Use also low complexity probes')
    parser.add_argument('--back', dest='back', type=int, action='store', default=None, help='the number of bases taken at the end of contigs to jump with pared-ends [default: <estimated>]')
    parser.add_argument('--snp', dest='snp', action='store_false', default=True, help='desactivate the SNP clearing mode')
    parser.add_argument('--logfile', dest='logfile', action='store_true', default=False, help='Create a logfile for the assembling')
    #parser.add_argument('--maxjump', dest='maxjump', type=int, action='store', default=1, help='COunt of pair end jump by extension branch')
    
    
    args = parser.parse_args()
    
    if args.logfile:
        logfile=open(args.outputFilename+'.log',"w")

    # Load the read index

    r = Index(args.indexFilename)
    
    # looks if seed protein are a file name or an internal list
    
    try:
        p = fasta(args.seedsFilenameAA)
        if args.logfile:
            print >>logfile,"Load protein sequences from file : %s" % args.seedsFilenameAA
    except IOError:
        p = getattr(orgasm.samples,args.seedsFilenameAA)
        if args.logfile:
            print >>logfile,"Load protein internal dataset : %s" % args.seedsFilenameAA
            
    if "matK" in p:
        gp = ['accD',
              'atpA','atpB','atpE','atpH',
              'ccsA',
              'cemA',
              'clpP',
              'matK',
              'ndhA',
              'petA','petG',
              'psbA','psbB','psbC','psbD','psbE','psbF','psbH','psbI','psbJ','psbK','psbL','psbM','psbN','psbT','psbZ',
              'psaA','psaB','psaC',
              'rbcL',
              'rpl12','rpl14','rpl16','rpl23','rpl32','rpl36',
              'rpoA','rpoB','rpoC1','rpoC2',
              "rps2","rps4","rps7",'rps8','rps12',"rps18",'rpl14',
              'ycf1',
              ]

        p=dict(x for x in p.items() if x[0] in gp)

    if args.logfile:
        logfile.flush()
    
                
    # looks if the internal blast was already run            
                
    try:
        # yes, load the previous results
        x = pickle.load(open(args.outputFilename+'.match.omx'))
        if args.logfile:
            print >>logfile,"Load protein match from previous run : %d matches" % sum(len(x[i]) for i in x)
    except: 
        # no, run it and save the results
        if args.logfile:
            print >>logfile,"Running protein matching against reads...",
            logfile.flush()
        x = r.lookForSeeds(p,mincov=1)
        if args.logfile:
            print >>logfile," : %d matches" % sum(len(x[i]) for i in x)
            logfile.flush()
        pickle.dump(x,open(args.outputFilename+'.match.omx',"w"))

    # First esti;ation of the coverage based on protein matches
    
    common    = set(p.keys()) & set(x.keys())
    pcoverage = [len(x[i]) * r.getReadSize()  / len(p[i]) / 3 for i in common]
    pcoverage.sort()

    print >>sys.stderr,pcoverage

    #coverage=mode(pcoverage)
    #coverage=pcoverage[len(pcoverage)/2]
    coverage=pcoverage[-1]
    
    # according to the minread option estimate it from coverage or use the specified value
   
    if args.minread is None:
        minread,minoverlap = estimateMinRead(r,args.minoverlap,coverage)
    else:
        minread = args.minread
        minoverlap = args.minoverlap

    print >>sys.stderr,"\nestimated coverage  : %d based on protein match (minread: %d)" % (coverage,minread)
    
    if args.logfile:
        print >>logfile,"\nestimated coverage  : %d based on protein match (minread: %d)" % (coverage,minread)
        logfile.flush()
        
        
    # Convert matches in sedd list    
    s = matchtoseed(x,r)
    
    # Create the assembler object
    asm = Assembler(r)

    ##########################
    #
    # If minread is not specified we initiate the assembling
    # based on the coverage estimated from protein match
    # to obtain a better coverage estimation
    #
    ##########################
    if args.minread is None:

        # Run the first assembling pass
        a = tango(asm,s,mincov=args.mincov,
                        minread=minread,
                        minoverlap=minoverlap,
                        lowfilter=not args.lowcomplexity,
                        maxjump=0, 
                        cycle=1,
                        nodeLimit=30000)
    
        # Clean small unsuccessful extensions
        asm.cleanDeadBranches(maxlength=10)
        
        # and too low covered assembling
        cutLowCoverage(asm,int(coverage/4),terminal=True)
    
        if args.snp:
            cutSNPs(asm)
        
        if args.smallbranches is not None:
            smallbranches = args.smallbranches
        else:
            smallbranches = estimateDeadBrancheLength(asm)
            print >>sys.stderr,"     Dead branch length setup to : %d bp" % smallbranches
            asm.cleanDeadBranches(maxlength=smallbranches)
        
        if len(asm) > 0:
            score,length,coverage = coverageEstimate(asm)    
            minread,minoverlap = estimateMinRead(r,args.minoverlap,coverage) 
            print >>sys.stderr,"coverage estimated : %d based on %d bp (minread: %d)" %(coverage,length,minread)
        
            
    
        s.extend(matchtoseed(x,r))

    #############################################
    #
    # We now run the main assembling process
    #
    #############################################

    # Run the first assembling pass
    a = tango(asm,s,mincov=args.mincov,
                    minread=minread,
                    minoverlap=minoverlap,
                    lowfilter=not args.lowcomplexity,
                    maxjump=0, 
                    cycle=1)
    
    # Clean small unsuccessful extensions
    asm.cleanDeadBranches(maxlength=10)
    
    # and too low covered assembling
    cutLowCoverage(asm,int(coverage/4),terminal=True)
    
    # cleanup snp bubble in the graph    
    if args.snp:
        cutSNPs(asm)
    
    if args.smallbranches is not None:
        smallbranches = args.smallbranches
    else:
        smallbranches = estimateDeadBrancheLength(asm)
        print >>sys.stderr,"     Dead branch length setup to : %d bp" % smallbranches

    asm.cleanDeadBranches(maxlength=smallbranches)


    # reestimate coverage
    score,length,coverage = coverageEstimate(asm)    
    
    if coverage < 30:
        sys.exit()

    if args.minread is None:
        minread,minoverlap = estimateMinRead(r,args.minoverlap,coverage)
        minread/=4
        if minread<5:
            minread=5

    print >>sys.stderr,"coverage estimated : %d based on %d bp (minread: %d)" %(coverage,length,minread)
    if args.logfile:
        print >>logfile,"coverage estimated : %d based on %d bp (minread: %d)" %(coverage,length,minread)
        logfile.flush()
        
    meanlength,sdlength = estimateFragmentLength(asm)
    
    if args.back is not None:
        back = args.back
    elif args.back is None and meanlength is not None:
        back = int(meanlength + 4 * sdlength)
        if back > 500:
            back=500
    else:
        back = 300
        
    if meanlength is not None:
        print >>sys.stderr,"Fragment length estimated : %f pb (sd: %f)" % (meanlength,sdlength)

    cg = asm.compactAssembling(verbose=False)
    scaffold(asm,cg,minlink=5,back=back,addConnectedLink=False)
    print >>open(args.outputFilename+'.intermediate.gml','w'),cg.gml()
    
    ###################################################
    #
    # We now fill the gaps between the contigs
    #
    ###################################################
    
    delta = 1
       
    # Run the fill gap procedure    
    while  delta > 0 or delta < -2000 :
                   
        delta = fillGaps(asm,back=back,
                       minread=minread,
                       maxjump=0,
                       minoverlap=minoverlap,
                       cmincov=0,
                       gmincov=int(coverage/4),
                       snp=args.snp)

        print >>sys.stderr
        print >>sys.stderr
        print >>sys.stderr
        print >>sys.stderr,"=================================================================="      
        print >>sys.stderr 
        
        # and too low covered assembling
        # cutLowCoverage(asm,int(coverage/3),terminal=True)
    
    
        # reestimate coverage
        score,length,coverage = coverageEstimate(asm)    
        if args.minread is None:
            minread,minoverlap = estimateMinRead(r,args.minoverlap,coverage)
            minread/=4
            if minread<5:
                minread=5

        print >>sys.stderr,"coverage estimated : %d based on %d bp (minread: %d)" %(coverage,length,minread)

        cg = asm.compactAssembling(verbose=False)
        scaffold(asm,cg,minlink=5,back=back,addConnectedLink=False)
        print >>open(args.outputFilename+'.intermediate.gml','w'),cg.gml()
        
        if meanlength is None:
            meanlength,sdlength = estimateFragmentLength(asm)
            if args.back is None and meanlength is not None:
                print >>sys.stderr,"Fragment length estimated : %f pb (sd: %f)" % (meanlength,sdlength)
                back = int(meanlength + 4 * sdlength)  
                if back > 500:
                    back=500
                         
        print >>sys.stderr
        print >>sys.stderr,"=================================================================="      
        print >>sys.stderr
        

    ###################################################
    #
    # Finishing of the assembling
    #
    ###################################################

    if args.snp:
        print >>sys.stderr,"Clean polymorphisms"
        cutSNPs(asm)
        
    asi = len(asm)+1
    
    print >>sys.stderr,"Clean dead branches"
    while (asi>len(asm)):
        asi=len(asm)
        smallbranches = estimateDeadBrancheLength(asm)
        print >>sys.stderr,"     Dead branch length setup to : %d bp" % smallbranches
        asm.cleanDeadBranches(maxlength=smallbranches)
        
    cg = asm.compactAssembling(verbose=False)
    score,length,coverage = coverageEstimate(asm)

    if args.snp:
        print >>sys.stderr,"Clean polymorphisms phase 2"
        cutSNPs(asm)
        
    print >>sys.stderr,"Clean low coverage terminal branches"
    cutLowCoverage(asm,int(coverage/2),terminal=True)
    print >>sys.stderr,"Clean low coverage internal branches"
    cutLowCoverage(asm,int(coverage/3),terminal=False)
        
    cg = asm.compactAssembling(verbose=False)     
        
    print >>sys.stderr,"Scaffold the assembly"
    scaffold(asm,cg,minlink=5,back=int(back),addConnectedLink=False)
    print >>open(args.outputFilename+'.gml','w'),cg.gml()

    print >>sys.stderr,"Evaluate pair-end constraints"
    cg = asm.compactAssembling(verbose=False)
    constraints = pathConstraints(asm,cg,back=int(back),minlink=5)
    scaffold(asm,cg,minlink=5,back=int(back),addConnectedLink=False)
     

    fastaout = open(args.outputFilename+'.fasta','w')
    
    
    print >>sys.stderr,"Select the good connected componantes"
    gcc = selectGoodComponent(cg)
    
    print >>sys.stderr,"Print the result as a fasta file"

    for seeds in gcc:
        path = unfoldAssembling(asm,cg,seeds=seeds,constraints=constraints)
        fa = path2fasta(asm,cg,path[-1][0],
             identifier=args.outputFilename,
             back=back,
             minlink=5)
        print >>fastaout,fa


