#!/usr/bin/env python
################################################
# setenv PYTHONPATH ./orgasm/build/lib.macosx-10.6-intel-2.7/
# echo $PYTHONPATH
# ./orgasm/build/lib.macosx-10.6-intel-2.7/
# python orgasmb.py ./samples/GWM-261 ./samples/COX1_Machaon.fasta machaon

from orgasm.tango import scaffold, \
                         path2fasta, \
                         unfoldAssembling, \
                         selectGoodComponent, \
                         pathConstraints, \
                         estimateFragmentLength, genesincontig
from orgasm.indexer import Index
from orgasm.tango import restoreGraph
                           
import sys
import pickle    

def estimateMinRead(index,minoverlap,coverage):
    MINREAD=10
    MINREADCOR=5
    MINOVERLAP=50
    minread =  (index.getReadSize() - minoverlap) * coverage / index.getReadSize()  / MINREADCOR
    if minread < MINREAD:
        minoverlap = index.getReadSize() - (MINREAD * MINREADCOR * index.getReadSize() / coverage)
        minread = MINREAD
    if  minoverlap< MINOVERLAP:
        minread =  MINREAD
        minoverlap = MINOVERLAP
    return minread,minoverlap

if __name__ == '__main__':
    
    import argparse
    
    # Declare script options
    
    parser = argparse.ArgumentParser(description='Assembly program dedicated to organel genome reconstruction.')
    parser.add_argument('indexFilename', metavar='index', help='index root filename (produced by orgasmi)')
    parser.add_argument('resultFilename', metavar='result', help='result prefix', default=None)    
    parser.add_argument('--back', dest='back', type=int, action='store', default=None, help='the number of bases taken at the end of contigs to jump with pared-ends [default: <estimated>]')
    
    args = parser.parse_args()
    
    if args.resultFilename is None:
        args.resultFilename=args.indexFilename
    
    # Load the read index

    r = Index(args.indexFilename)
    x = pickle.load(open(args.resultFilename+'.omx'))

    asm = restoreGraph(args.resultFilename+'.oax',r,x)

    print >>sys.stderr,"Evaluate fragment length"
    meanlength,sdlength = estimateFragmentLength(asm)
    
    if meanlength is not None:
        print >>sys.stderr,"Fragment length estimated : %f pb (sd: %f)" % (meanlength,sdlength)

    if args.back is not None:
        back = args.back
    elif args.back is None and meanlength is not None:
        back = int(meanlength + 4 * sdlength)
        if back > 500:
            back=500
    else:
        back = 300

    print >>sys.stderr,"Evaluate pair-end constraints"
    cg = asm.compactAssembling(verbose=False)
    genesincontig(cg,r,x)

    constraints = pathConstraints(asm,cg,back=int(back),minlink=5)
    scaffold(asm,cg,minlink=5,back=int(back),addConnectedLink=False)
     

    fastaout = sys.stdout
    
    
    print >>sys.stderr,"Select the good connected componantes"
    gcc = selectGoodComponent(cg)
    
    print >>sys.stderr,"Print the result as a fasta file"


    c=1
    for seeds in gcc:
        path = unfoldAssembling(asm,cg,seeds=seeds,constraints=constraints)
                
        fa = path2fasta(asm,cg,path[-1][0],
             identifier="Seq_%d" % c,
             back=back,
             minlink=5)
        print >>fastaout,fa
        c+=1
        
    print >>open(args.resultFilename +'.gml','w'),cg.gml()
    


